@@ -4,12 +4,14 @@ use strict;
 use warnings;
 use base qw( Exporter );
 use File::Spec;
+use File::Which qw( which );
+use File::Basename qw( dirname );
 
 our @EXPORT    = qw( msys msys_run );
 our @EXPORT_OK = qw( msys msys_run msys_path );
 
 # ABSTRACT: Tools required for GNU style configure scripts on Windows
-our $VERSION = '0.10_01'; # TRIAL VERSION
+our $VERSION = '0.10_02'; # TRIAL VERSION
 $VERSION = eval $VERSION;
 
 
@@ -34,6 +36,8 @@ sub msys_run
 }
 
 
+our $location_method = '-';
+
 sub msys_path ()
 {
   return undef unless  $^O eq 'MSWin32';
@@ -42,15 +46,24 @@ sub msys_path ()
   
   if($override_type ne 'share')
   {
-    return $ENV{PERL_ALIEN_MSYS_BIN}
-      if defined $ENV{PERL_ALIEN_MSYS_BIN} && -x File::Spec->catfile($ENV{PERL_ALIEN_MSYS_BIN}, 'sh.exe');
-  
-    require File::Spec;
-    foreach my $dir (split /;/, $ENV{PATH})
+    if(defined $ENV{PERL_ALIEN_MSYS_BIN} && -x File::Spec->catfile($ENV{PERL_ALIEN_MSYS_BIN}, 'sh.exe'))
+    {
+      $location_method = 'PERL_ALIEN_MSYS_BIN';
+      return $ENV{PERL_ALIEN_MSYS_BIN}
+    }
+
+    if(my $uname_exe = which('uname'))
+    {
+      my $uname = `$uname_exe`;
+      if($uname =~ /^(MSYS|MINGW(32|64))_NT/) {
+        $location_method = 'found msys2';
+        return dirname($uname_exe);
+      }
+    }
+
+    if(my $mingw_get = which('mingw-get.exe'))
     {
       my $path = eval {
-        my $mingw_get = File::Spec->catfile($dir, 'mingw-get.exe');
-        die 'no mingw-get.exe' unless -x $mingw_get;
         my($volume, $dirs) = File::Spec->splitpath($mingw_get);
         my @dirs = File::Spec->splitdir($dirs);
         splice @dirs, -2;
@@ -59,12 +72,20 @@ sub msys_path ()
         die 'no sh.exe' unless -x File::Spec->catfile($path, 'sh.exe');
         $path;
       };
-      return $path unless $@;
+      unless($@)
+      {
+        $location_method = 'relative to mingw-get.exe';
+        return $path;
+      }
     }
 
     foreach my $dir (qw( C:\MinGW\msys\1.0\bin ))
     {
-      return $dir if -x File::Spec->catfile($dir, 'sh.exe');
+      if(-x File::Spec->catfile($dir, 'sh.exe'))
+      {
+        $location_method = 'default install location';
+        return $dir;
+      }
     }
 
     my $path = eval {
@@ -83,13 +104,21 @@ sub msys_path ()
       $path;
     };
 
-    return $path unless $@;
+    unless($@)
+    {
+      $location_method = 'desktop shortcut';
+      return $path;
+    }
   }
 
   if($override_type ne 'system')
   {
     my $dir = _my_dist_dir();
-    return $dir if defined $dir && -d $dir;
+    if($dir && -d $dir)
+    {
+      $location_method = 'default install location';
+      return $dir if defined $dir && -d $dir;
+    }
   }
 
   return undef;
@@ -129,7 +158,7 @@ Alien::MSYS - Tools required for GNU style configure scripts on Windows
 
 =head1 VERSION
 
-version 0.10_01
+version 0.10_02
 
 =head1 SYNOPSIS
 
@@ -190,7 +219,7 @@ L<Win32::Shortcut> installed, as it is an optional dependency.
 =back
 
 If C<MSYS> cannot be found using any of these methods, then it will download and install
-C<MSYS> in this distribution's share directory (via L<File::ShareDir>).
+C<MSYS> in this distribution's share directory.
 
 =head1 FUNCTIONS
 
